/*
** Module   : BufferValidator.java
** Abstract : provides a validation callback mechanism for buffers
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 GES 20130224 Provides a validation callback mechanism for buffer operations.
** 002 VMN 20130729 Error messages correction for validate().
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Provides a validation callback mechanism which allows an arbitrary user
 * defined expression to be processed by the persistence framework during
 * language statement processing.  Some operation on a table/buffer can be
 * aborted if the expression does not evaluate to <code>true</code>.
 * <p>
 * To use this class, create a subclass of this class and override the
 * {@link #validateExpression} and {@link #validateMessage} methods.
 * <p>
 * The <code>validateExpression</code> method can have any arbitrary
 * logic or processing and on failure it must return <code>false</code>.
 * This method will be called by the {@link #validate} method of this class. 
 * <p>
 * The <code>validateMessage</code> method can have any arbitrary logic or
 * processing and it is only called by the {@link #validate} method of this
 * class.  It must return the error message to be displayed to the user 
 * when a validation failure occurs (when the <code>validateExpression</code>
 * returns <code>false</code>.
 * <p>
 * At runtime, the <code>validate</code> method first calls
 * <code>validateExpression</code> and on failure it then throws an
 * {@link ErrorConditionException} with the message text of the error set to
 * the returned value of the <code>validateMessage</code> method.
 * <p>
 * To make arbitrary expressions evaluate properly, any references to 
 * objects/resources in the scope of where the sub-class is defined
 * (usually as an anonymous local class) cannot be non-final local variables 
 * or non-final method parameters, but rather they must be static or instance
 * members or final variables/parameters.  This allows the embedded
 * expression methods to dereference these elements to obtain runtime data.
 * In particular, any non-final local variables or non-final method
 * parameters are inaccessible since an anonymous local class may not
 * directly access non-final local references.
 */
public abstract class BufferValidator
{
   /**
    * Processes validation rules, error checking and any other user-defined
    * constraint processing that needs to be applied.  This is used
    * to check if a proposed update to an object will pass validation. If
    * the method silently returns then all tests have passed, otherwise
    * an exception will be thrown on any validation failure.
    *
    * @return   <code>true</code> if the validation succeeded and the record can be
    *           deleted.
    *
    * @throws   ErrorConditionException
    *           On the failure of validation processing or other error
    *           checking.
    */
   public boolean validate()
   throws ErrorConditionException
   {
      boolean ok;
      try
      {
         ok = validateExpression().booleanValue();
      }
      catch (ErrorConditionException e)
      {
         ErrorManager.recordOrThrowError(0, validateMessage().toStringMessage(), false, true);
         throw new EndConditionException(e);
      }

      // TODO: is this the right behavior for buffer validation?
      if (!ok)
      {
         // check IF this even generates an error condition AND what the err num should
         // be if it does AND IF it honors NO-ERROR (which definitely can be applied to
         // the delete statement)
         ErrorManager.recordOrThrowError(0, validateMessage().toStringMessage(), false, true);
      }
      
      return ok;
   }
   
   /**
    * User-defined expression which is evaluated to test if a proposed buffer
    * operation is valid.  It is possible that the validation or error checking
    * can generate an error condition directly though usually this is done by
    * the calling method {@link #validate}.
    *
    * @return   <code>true</code> if the validation passes and <code>false</code> on any
    *           failure.
    *
    * @throws   ErrorConditionException
    *           If any failure occurs during expression processing.
    */
   public abstract logical validateExpression()
   throws ErrorConditionException;
   
   /**
    * User-defined expression which generates the message text to display to
    * the user when the proposed buffer operation is known to be invalid.  It is
    * possible that the message expression evaluation can generate an error
    * condition directly though usually this is done by the calling method
    * {@link #validate} using the returned text as the message text of the
    * exception.
    *
    * @return   The message to display to the user.
    *
    * @throws   ErrorConditionException
    *           If any failure occurs during expression processing.
    */
   public abstract character validateMessage()
   throws ErrorConditionException;   
}

